# Magic Leap MRTK3 Package

This package adds support for Magic Leap devices when using the
Mixed Reality Toolkit (MRTK) version 3.

## Current Status

| Feature | Status |
|--|--|
| Controller | Pre-Release |
| Hand Tracking | Pre-Release |

### Important Note

This package is an early access version. This means features are still in active development and subject to changes or even having their implementation completely removed and done in a different way.

**Please use the Unity SDK 1.4 Package with the MRTK3 preview package at this time.**

## Prerequisites

- Magic Leap SDK v1.2.0
- Magic Leap Unity SDK v1.4.0

This setup guide assumes you have already configured your Unity Project for ML2. (https://developer.magicleap.cloud/learn/docs/guides/unity/getting-started/configure-unity-settings)

**Please Note there is a Magic Leap Setup Tool to make this process easier, but at this time it only downloads the latest Unity SDK and we need the specific 1.4 version. In the future this tool will allow for importing different SDK versions.**

## Getting Started

Before importing the Magic Leap MRTK v3 Early Access Package, developers will need to configure their project for MRTK v3. This section provides general guidance on downloading and installing the MRTK v3 packages using the Mixed Reality Feature tool(Windows Only) or using the MRTK v3 Template Project.

### Creating a new MRTK V3 Project

Download the MRTK dependencies using one of two methods: Using the [MixedReality Feature Tool](https://learn.microsoft.com/en-us/windows/mixed-reality/develop/unity/welcome-to-mr-feature-tool) (Windows Only), Expanded on below, or by manually downloading and importing the packages from [Github](https://github.com/microsoft/MixedRealityToolkit-Unity/tree/mrtk3). MRTK Input and MRTK UX Components packages are required for using with Magic Leap, please review [MRTK Package Dependencies](https://learn.microsoft.com/en-us/windows/mixed-reality/mrtk-unity/mrtk3-overview/packages/packages-overview) to know which dependencies are required for those two packages.

#### Using the Mixed Reality Feature Tool

This section provides instructions on installing the MRTK v3 dependencies into an existing project using the [Mixed Reality Feature tool](https://learn.microsoft.com/en-us/windows/mixed-reality/develop/unity/welcome-to-mr-feature-tool). Note this tool is only available for Windows.

1. Open the Mixed Reality Feature tool
2. Target your Unity project.
3. At a minimum install the following packages, as they are required:

- MRTK3 / MRTK Input
- MRTK3 / MRTK UX Components

*Note: If you do not see MRTK v3, you may need to select the **Show preview releases** option located at the bottom of the window.*

4. After choosing the the packages to install select **Get Features**. This will display the package dependencies.
5. Finally, press **Import** then **Approve**
6. Clear any errors that appear as a result of a missing dependency from a prefab of XR provider.

To learn more see Microsoft's [Starting from a new project](https://learn.microsoft.com/en-us/windows/mixed-reality/mrtk-unity/mrtk3-overview/getting-started/setting-up/setup-new-project) guide.

### Using the MRTK3 Template Project

You can download the MRTK packages, and explore MRTK v3's features more quickly by downloading the MRTK project from the `mrtk3` branch in the [MRTK Github repository](https://github.com/microsoft/MixedRealityToolkit-Unity/tree/mrtk3). To use the Template project, clone MRTK from the GitHub repo and check out the `mrtk3` branch. [Official Microsoft Guide Here] (https://learn.microsoft.com/en-us/windows/mixed-reality/mrtk-unity/mrtk3-overview/getting-started/setting-up/setup-template)

If you work with Git using the command line, you can clone the repo while specifying the mrtk3 branch: `git clone --branch mrtk3 https://github.com/microsoft/MixedRealityToolkit-Unity.git`

Once the Github project is downloaded, follow the steps below to update the project settings to be compatible with Magic Leap 2.

#### Template Project Setup

1. Using the Unity Hub, open the `MRTKDevTemplate` project using Unity version 2022.2.x
   1. On the **Opening Project in Non-Matching Editor Installation** popup select **Continue**.
   2. On the **Script Updating Consent** popup select **Yes, for these and other files that might be found later**.
   3. On the **Enter Safe Mode?** popup select **Ignore**.
2. Open the package manager (Window > Package Manager) and remove the following packages
   1. **Project Auditor**
   2. **Mixed Reality OpenXR Plugin**
   3. **Google ARCore XR Plugin**
   4. **OpenXR Plugin**
3. Clear any errors that appear as a result of a missing dependency from a prefab of XR provider.
   1. If Errors are still present, close the project and delete the project's Library folder and re-open unity to reimport the existing packages.
4. Download and Install the [Magic Leap Setup Tool](https://assetstore.unity.com/packages/tools/integration/magic-leap-setup-tool-194780) from the Unity Asset store.
5. Once installed, use the Project Setup window to configure your project settings. Complete all the steps in the project setup tool.

## Importing MRTK v3 Magic Leap

Once the project is configured for ML2 and has the required MRTK3 packages, import the provided MRTK3 Magic Leap package into the project.

1. Open the Package Manager (**Window** > **Package Manager**) and import the `com.magicleap.mrtk3.tgz` package. Select the **＋** icon then select **Add package from tarball...**

2. In Open MRTK3's Project settings (**Edit** > **Project Settings** > **MRTK3**). Then set the **Profile** to **MRTKProfile - MagicLeap**.

3. Remove and Replace the standard XR Rig from the Scene with the `MRTK XR Rig - MagicLeap` prefab variant located in `Packages/com.magicleap.mrtk3/Runtime/MagicLeap/Prefabs/MRTK_Variants/`

*See the [Configure Existing Rig](#configuring-an-existing-xr-rig) section below on information on how to edit the default MRTK rig instead of replacing it.*

### Configuring an existing XR Rig

This section describes how to configure MRTKs existing XR rig so instead of replacing it with the pre-configured "MRTK XR Rig - MagicLeap" variant.

1. Select the **MRTK XR Rig** in the scene. Add the **MagicLeapInputs** and **MagicLeapAuxiliaryInputs** input action assets to the **Input Action Manager**
2. Expand object so that the MRTK RightHand and MRTK LeftHand Controllers are visible (`MRTK XR Rig/ Camera Offset/`).
3. Add the  `Packages/com.magicleap.mrtk3/Runtime/MagicLeap/Prefabs/MRTK_Variants/MRTK MagicLeap Controller` to add support for MagicLeap controller input.
4. Select each of the Hand Controller objects and update the following components.
   1. Update the **Articulated Hand Controller** to use equivalent `MagicLeapAuxiliaryInputs` inputs instead of the generic MRTK bindings. For example, the left hand bindings would be the following in order:
      1. `Aux LeftHand/DevicePosition`
      2. `Aux LeftHand/DeviceRotation`
      3. `Aux LeftHand/TrackingState`
      4. `Aux LeftHand/Select`
      5. `Aux LeftHand/Select Value`
   2. Remove actions that do not have matching values. For the left hand these values would be:
       1. `MRTK LeftHand/Activate`
       2. `MRTK LeftHand/UI Press`
       3. `MRTK LeftHand/Rotate Anchor`
       4. `MRTK LeftHand/Translate Anchor`
5. Replace the input binding on each of the hand's child objects so they target the `MagicLeapAuxiliaryInputs` input actions.
   1. Select the **IndexTip PokeInteractor** then expand the Poke Pose Source/Pose Source List.
    Replace Element 1, to use Tracking State and PointerPosition/Rotation actions.
   2. Select the **Far Ray** then select the Aim Pose Source Pose / Source List. Replace Element 0, to use Tracking State and PointerPosition/PointerRotation actions.
   3. Select the **Far Ray** hen select the Device Pose Source / Pose Source List. Replace Element 0 to use Tracking State and DevicePosition/DeviceRotation actions.
   4. Select **GrabInteractor** then select the Pinch Pose Source / Pose Source List. Replace Element 1 Tracking State and PointerPosition/PointerRotation actions.
   5. Select the **GazePinchInteractor** and Replace the **Device Pose Source**   (DevicePosition/Rotation), and **Aim Pose Source** (PointerPosition/Rotation).
6. Finally, select the **Main Camera**, then add the **MagicLeap Camera** component. *(Optional)*

## FAQ

Why do I need to replace the rig?
Replacing the rig is a temporary requirement to make sure the Magic Leap input bindings target the standard MRTK inputs.
